(*********************************************************************************)
(*                OWS                                                            *)
(*                                                                               *)
(*    Copyright (C) 2003 Institut National de Recherche en Informatique et       *)
(*    en Automatique. All rights reserved.                                       *)
(*                                                                               *)
(*    This program is free software; you can redistribute it and/or modify       *)
(*    it under the terms of the GNU General Public License as published by       *)
(*    the Free Software Foundation; either version 2 of the License, or          *)
(*    any later version.                                                         *)
(*                                                                               *)
(*    This program is distributed in the hope that it will be useful,            *)
(*    but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *)
(*    GNU General Public License for more details.                               *)
(*                                                                               *)
(*    You should have received a copy of the GNU General Public License          *)
(*    along with this program; if not, write to the Free Software                *)
(*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA                   *)
(*    02111-1307  USA                                                            *)
(*                                                                               *)
(*    Contact: Maxence.Guesdon@inria.fr                                          *)
(*                                                                               *)
(*********************************************************************************)

(** OWS (OCaml Web Sessions) interface. *)


(** A page name is a string, with only the following characters:
 [a-z A-Z 0-9 '-' '_']*)
type page = string

(** A session identifier is a string, with only the following characters:
 [a-z A-Z 0-9 '-' '_']*)
type session_id = string

(** The host accessing a page. *)
type hostname = string

(** This exception can be used to exit from
   a page handler with a document.*)
exception Exit_doc of Toolhtml.doc

(** This exception can be used to exit from a page handler,
   and to use another handler with the same host and optional user info,
   but with the gievn arguments. *)
exception Exit_handler of string * (string * string) list

(** What we need to manage sessions. *)
module type Conf = sig
  (** to identify a user in the handler functions *)
  type t 

  (** To return a document from an error message, when
     a page handler raises the [Failure] exception. *)
  val failure_handler : string -> Toolhtml.doc

  (** to get a user id from the session id and the host name *)
  val get_user_id : session_id -> hostname -> t option

  (** the variable name to use for the page in urls,
     with only the following characters: [a-z A-Z 0-9 '-' '_']*)
  val page_varname : string

  (** the variable name to use for the session id in urls,
     with only the following characters: [a-z A-Z 0-9 '-' '_']*)
  val session_varname : string 

end

(** A functor to create the module with the functions 
   to manage sessions between pages in the parameters 
   (URLs or Forms). *)
module Params_based : functor (C : Conf) ->
  sig
    type user_id = C.t

    (** An argument for a page *)
    type arg = string * string

    (** A handler is a function to call for a given page.
       The function takes 
       - the hostname of the client,
       - an option pair [(user_id, sid)] if the user is identified (with a session),
       then [sid] is the sessions id,
       - the list of arguments of the page.

       Then the function returns an HTML document.
    *)
    type handler = hostname -> (user_id * string) option -> arg list -> Toolhtml.doc
    
    (** Register a page handler. *)
    val register : page -> handler -> unit

    (** Take a session id and returns the code to append to an url to use
       the same session id for the page at this url.*)
    val arg_of_session_id : session_id -> string

    (** Take a page name and build the code to append to an url
       to go to this page. *)
    val arg_of_page : page -> string

    (** Return the cgi-bin url. *)
    val cgibin : unit -> string

    (** Take a page name a return the cgi-bin url to go to that page. *)
    val cgibin_of_page : page -> string

    (** Handle the required page (according to the value of the variable for the page
       in the url. This is the function to call at the end of the program.
       The default handler is given in argument.
    *)
    val handle : handler -> unit
  end

(** To set or unset a pair (name, value) in a cookie. *)
type cookie_action =
    Set of string * string 
  | Unset of string * string


(** A functor to create the module with the functions 
   to manage sessions between pages using cookies.*)
module Cookies_based : functor (C : Conf) ->
  sig
    type user_id = C.t

    (** An argument for a page *)
    type arg = string * string

    (** A handler is a function to call for a given page.
       The function takes 
       - the hostname of the client,
       - an option pair [(user_id, sid)] if the user is identified 
         (with a session), then [sid] is the sessions id,
       - the list of arguments of the page.

       Then the function returns an HTML document.
    *)
    type handler = hostname -> (user_id * string) option -> arg list -> Toolhtml.doc
    
    
    (** The cookies to set or unset in the reply. *)
    val cookies_actions : cookie_action list ref

    (** Register a page handler. *)
    val register : page -> handler -> unit

    (** Take a page name and build the code to append to an url
       to go to this page. *)
    val arg_of_page : page -> string

    (** Return the cgi-bin url. *)
    val cgibin : unit -> string

    (** Take a page name a return the cgi-bin url to go to that page. *)
    val cgibin_of_page : page -> string

    (** Handle the required page (according to the value of the variable for the page
       in the url. This is the function to call at the end of the program.
       The default handler is given in argument.
    *)
    val handle : handler -> unit
  end

